package com.channon.android.androframe_ads;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ExpandableListActivity;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ExpandableListView;
import android.widget.SimpleExpandableListAdapter;

@SuppressWarnings({ "rawtypes", "unchecked" })
public class InfoActivity extends ExpandableListActivity
{
	    private static final String LOG_TAG = "androframe";
	    
	  //List Choices
		public static final int CHOICE_GENERAL = 0;
		public static final int CHOICE_IMGCHOOSER = 1;
		public static final int CHOICE_PREFS = 2;
		public static final int CHOICE_WIDGET = 3;
		public static final int CHOICE_FEEDBACK = 9;

	    static final String sections[] = {
		  "Information",
		  "Add-Ons",
		  "Feedback",
		};

		static final String members[][] = {
	// Members of Info
		  {
			"General","How AndroFrame works",
			"Images Chooser","How to use the image chooser",
			"Preferences","How to configure the app",
			"Widget", "How to use the widget",
		  },
	// Members of Add-Ons
		  {
			"AndroFrame Full Version","Open the Android Market",
		  },
	// Members of Feedback
		  {
			"Email","Click to email feedback to the developer",
		  },
	
	    };

	    /** Called when the activity is first created. */
	    @Override
	    public void onCreate(Bundle icicle)
	    {
	        super.onCreate(icicle);
	        setContentView(R.layout.infoactivity_layout);
			SimpleExpandableListAdapter expListAdapter =
				new SimpleExpandableListAdapter(
					this,
					createGroupList(),	// groupData describes the first-level entries
					R.layout.info_group_row,	// Layout for the first-level entries
					new String[] { "sectionName" },	// Key in the groupData maps to display
					new int[] { R.id.childname },		// Data under "colorName" key goes into this TextView
					createChildList(),	// childData describes second-level entries
					R.layout.info_child_row,	// Layout for second-level entries
					new String[] { "memberName", "text" },	// Keys in childData maps to display
					new int[] { R.id.childname, R.id.text }	// Data under the keys above go into these TextViews
				);
			setListAdapter( expListAdapter );
	    }

	    @Override
		public void  onContentChanged  () {
	        super.onContentChanged();
	        Log.d( LOG_TAG, "onContentChanged" );
	    }

	    @Override
		public boolean onChildClick(
	            ExpandableListView parent, 
	            View v, 
	            int groupPosition,
	            int childPosition,
	            long id) {
	        Log.d( LOG_TAG, "onChildClick: "+childPosition );
	        
	        //CheckBox cb = (CheckBox)v.findViewById( R.id.check1 );
	        //if( cb != null )
	        //    cb.toggle();

	        
	        switch(groupPosition) {
	        	case 0:
	        		showDialog(childPosition);
	        		break;
	        	case 1:
	        		Intent intent = new Intent(Intent.ACTION_VIEW);
	        		intent.setData(Uri.parse("market://details?id=com.channon.android.androframe"));
	        		startActivity(intent);
	        		break;
	        	case 2:
	        		final Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);
	  	  			emailIntent.setType("plain/text");
	  	  			emailIntent.putExtra(android.content.Intent.EXTRA_EMAIL, new String[] { "com.channon.android@gmail.com" });
	  	  			emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, "AndroFrame Feedback");
	  	  			
	  	  			//ilIntent.putExtra(android.content.Intent.EXTRA_TEXT, stacktrace);
	  	  	
	  	  			// START THE EMAIL ACTIVITY - NOTE YOU NEED TO START IT WITH A CHOOSER
	  	  	        
	  	  			startActivity(Intent.createChooser(emailIntent, "Send feedback..."));
	  	  			break;
	        		
	        }
	        
	        return false;
	    }

	    @Override
		public void  onGroupExpand  (int groupPosition) {
	        Log.d( LOG_TAG,"onGroupExpand: "+groupPosition );
	    }

	/**
	  * Creates the group list out of the colors[] array according to
	  * the structure required by SimpleExpandableListAdapter. The resulting
	  * List contains Maps. Each Map contains one entry with key "colorName" and
	  * value of an entry in the colors[] array.
	  */
		
		private List createGroupList() {
		  ArrayList result = new ArrayList();
		  for( int i = 0 ; i < sections.length ; ++i ) {
			HashMap m = new HashMap();
		    m.put( "sectionName",sections[i] );
			result.add( m );
		  }
		  return result;
	    }

	/**
	  * Creates the child list out of the shades[] array according to the
	  * structure required by SimpleExpandableListAdapter. The resulting List
	  * contains one list for each group. Each such second-level group contains
	  * Maps. Each such Map contains two keys: "shadeName" is the name of the
	  * shade and "rgb" is the RGB value for the shade.
	  */
	  private List createChildList() {
		ArrayList result = new ArrayList();
		for( int i = 0 ; i < members.length ; ++i ) {
	// Second-level lists
		  ArrayList secList = new ArrayList();
		  for( int n = 0 ; n < members[i].length ; n += 2 ) {
		    HashMap child = new HashMap();
			child.put( "memberName", members[i][n] );
		    child.put( "text", members[i][n+1] );
			secList.add( child );
		  }
		  result.add( secList );
		}
		return result;
	  }
	  
	  @Override
		 protected Dialog onCreateDialog(int id) {
		        switch (id) {
		        case CHOICE_GENERAL:
		        	 return new AlertDialog.Builder(InfoActivity.this)
		                .setIcon(R.drawable.alert_dialog_icon)
		                .setTitle(R.string.info_images_title)
		                .setMessage(R.string.info_images_msg)
		                .setPositiveButton(R.string.alert_dialog_ok, new DialogInterface.OnClickListener() {
		                    public void onClick(DialogInterface dialog, int whichButton) {
		    
		                        /* User clicked OK so do some stuff */
		                    }
		                })
		                .create();
		        case CHOICE_IMGCHOOSER:
		        	 return new AlertDialog.Builder(InfoActivity.this)
		                .setIcon(R.drawable.alert_dialog_icon)
		                .setTitle(R.string.info_imgchs_title)
		                .setMessage(R.string.info_imgchs_msg)
		                .setPositiveButton(R.string.alert_dialog_ok, new DialogInterface.OnClickListener() {
		                    public void onClick(DialogInterface dialog, int whichButton) {
		    
		                        /* User clicked OK so do some stuff */
		                    }
		                })
		                .create();
		        case CHOICE_PREFS:
		  	  		 return new AlertDialog.Builder(InfoActivity.this)
		             .setIcon(R.drawable.alert_dialog_icon)
		             .setTitle(R.string.info_prefs_title)
		             .setMessage(R.string.info_prefs_msg)
		             .setPositiveButton(R.string.alert_dialog_ok, new DialogInterface.OnClickListener() {
		                 public void onClick(DialogInterface dialog, int whichButton) {
		 
		                     /* User clicked OK so do some stuff */
		                 }
		             })
		             .create();
	  	  		case CHOICE_WIDGET:
		  	  		 return new AlertDialog.Builder(InfoActivity.this)
		             .setIcon(R.drawable.alert_dialog_icon)
		             .setTitle(R.string.info_widget_title)
		             .setMessage(R.string.info_widget_msg)
		             .setPositiveButton(R.string.alert_dialog_ok, new DialogInterface.OnClickListener() {
		                 public void onClick(DialogInterface dialog, int whichButton) {
		 
		                     /* User clicked OK so do some stuff */
		                 }
		             })
		             .create();
	  	  		case CHOICE_FEEDBACK:
	  	  			
	  	  		final Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);
	  	  			emailIntent.setType("plain/text");
	  	  			emailIntent.putExtra(android.content.Intent.EXTRA_EMAIL, new String[] { "com.channon.android@gmail.com" });
	  	  			emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, "AndroFrame Feedback");
	  	  			
	  	  			//ilIntent.putExtra(android.content.Intent.EXTRA_TEXT, stacktrace);
	  	  	
	  	  			// START THE EMAIL ACTIVITY - NOTE YOU NEED TO START IT WITH A CHOOSER
	  	  	        
	  	  			startActivity(Intent.createChooser(emailIntent, "Send feedback..."));

	  	  			break;
	  	  		}
		        
				return null;
		 }

}