package com.channon.android.androframe_ads.objects;

import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Random;
import java.util.Set;

import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.os.Bundle;
import android.util.Log;

import com.channon.android.androframe_ads.AndroFrame;

public class DatabaseService {
	
	private DatabaseHelper dbHelper;
	private static SQLiteDatabase database;
	
	private String current_wp = null;
	
	// Database fields
	public static final String KEY_ROWID = "_id";
	public static final String KEY_PATH = "path";
	public static final String KEY_URI_ID = "uri_id";
	public static final String KEY_W_ID = "w_id";
	public static final String KEY_KEY = "setting_key";
	public static final String KEY_VALUE = "setting_value";
	public static final String KEY_FOLDER = "folder";
	
	
	//public static final String KEY_DESCRIPTION = "description";
	public static final String DATABASE_TABLE1 = "androframe_selections";
	public static final String DATABASE_TABLE2 = "androframe_history";
	public static final String DATABASE_TABLE3 = "androframe_settings";
	public static final String DATABASE_TABLE4 = "widget_settings";
	public static final String DATABASE_TABLE5 = "selection_settings";
	public static final String DATABASE_TABLE6 = "selection_folders";
	
	private final static String LOGTAG = "com.channon.android.androframe.DatabaseService";
	
	public DatabaseService() {
		super();
		Log.i(LOGTAG,"databaseservice created");
		
		dbHelper = new DatabaseHelper(AndroFrame.getInstance());
		database = dbHelper.getWritableDatabase();

	}

	/**
	 * Deletes row with matching path
	 */
	public void delete(String path) {
		
		final String encodedPath = URLEncoder.encode(path);
		
		database.execSQL("DELETE FROM "+ DATABASE_TABLE1 + " WHERE "+KEY_PATH+" = '"+encodedPath+"'");
		return;
	}
	
	/**
	 * Deletes row with matching path
	 */
	public void deleteAll() {
		database.execSQL("DELETE FROM "+ DATABASE_TABLE1);
		return;
	}
	
	/**
	 * Fethches all rows
	 * 	  
	 */
	public Cursor fetchAll() {
		Cursor cursor = database.rawQuery("SELECT "+KEY_ROWID+", "+KEY_PATH+", "+KEY_URI_ID+","+KEY_FOLDER+" FROM "+DATABASE_TABLE1+"",null);
		
		Log.i(LOGTAG,"fetchall cursor count: "+cursor.getCount());
		
		return cursor;
		
		//return database.query(DATABASE_TABLE, new String[] { KEY_ROWID, KEY_URI, KEY_FLAGS}, null, null, null, null, null);
	}
	
	public Cursor fetchFolder(String folder) {
		Cursor cursor = database.rawQuery("SELECT "+KEY_ROWID+", "+KEY_PATH+", "+KEY_URI_ID+","+KEY_FOLDER+" FROM "+DATABASE_TABLE1+" WHERE "+KEY_FOLDER+" = '"+folder+"' ",null);
		
		Log.i(LOGTAG,"fetchall cursor count: "+cursor.getCount());
		
		return cursor;
		
		//return database.query(DATABASE_TABLE, new String[] { KEY_ROWID, KEY_URI, KEY_FLAGS}, null, null, null, null, null);
	}
	
	public String getAppPref(String key) {
		Cursor cursor = database.rawQuery("SELECT "+KEY_KEY+","+KEY_VALUE+" FROM "+DATABASE_TABLE3+" WHERE "+KEY_KEY+" = '"+key+"' ",null);
		
		if(cursor.getCount()<=0){
			return null;
		}
		cursor.moveToFirst();
		int value_col = cursor.getColumnIndex(KEY_VALUE);
		return cursor.getString(value_col);

	}
	
	public void setAppPref(String key, String value) {
		Log.i("anroframe","updating preference: "+key+" with value: "+value+" for app");
		
		String sql = "DELETE FROM "+ DATABASE_TABLE3 + " WHERE "+KEY_KEY+" = '"+key+"' ";
		
		Log.i(LOGTAG,"sql: "+sql);
		
		database.execSQL(sql);
		
		sql = "REPLACE INTO "+ DATABASE_TABLE3 + " ("+KEY_KEY+","+KEY_VALUE+") VALUES ('"+key+"','"+value+"') ";
		
		Log.i(LOGTAG,"sql: "+sql);
		
		database.execSQL(sql);

	}
	
	public void setWPref(int appWidgetId, String key, String value) {
		Log.i("anroframe","updating preference: "+key+" with value: "+value+" for app");
		
		String sql = "DELETE FROM "+ DATABASE_TABLE4 + " WHERE "+KEY_KEY+" = '"+key+"' and "+KEY_W_ID+" = "+appWidgetId+" ";
		
		Log.i(LOGTAG,"sql: "+sql);
		
		database.execSQL(sql);
		
		sql = "REPLACE INTO "+ DATABASE_TABLE4 + " ("+KEY_W_ID+","+KEY_KEY+","+KEY_VALUE+") VALUES ("+appWidgetId+",'"+key+"','"+value+"') ";
		
		Log.i(LOGTAG,"sql: "+sql);
		
		database.execSQL(sql);

	}
	
	public String getWPref (int appWidgetId, String key) {
		Cursor cursor = database.rawQuery("SELECT "+KEY_KEY+","+KEY_VALUE+" FROM "+DATABASE_TABLE4+" WHERE "+KEY_W_ID+" = "+appWidgetId+" AND "+KEY_KEY+" = '"+key+"' ",null);
		
		if(cursor.getCount()<=0){
			return null;
		}
		cursor.moveToFirst();
		int value_col = cursor.getColumnIndex(KEY_VALUE);
		return cursor.getString(value_col);
	}
	
	public Bundle getWPrefs (int appWidgetId) {
		Cursor cursor = database.rawQuery("SELECT "+KEY_KEY+","+KEY_VALUE+" FROM "+DATABASE_TABLE4+" WHERE "+KEY_W_ID+" = "+appWidgetId+" ",null);
		Bundle values = new Bundle();
		
		//values.put("appWidgetId", String.valueOf(appWidgetId));
		
		
		if(cursor.getCount()>0) {
			int key_col = cursor.getColumnIndex(KEY_KEY);
			int value_col = cursor.getColumnIndex(KEY_VALUE);
			
			for(int i = 0; i < cursor.getCount(); i++) {
				cursor.moveToPosition(i);
				Log.i(LOGTAG,"retrieving preference: "+cursor.getString(key_col)+" with value: "+ cursor.getString(value_col)+" for widget: " + appWidgetId);
				
				values.putString(cursor.getString(key_col), cursor.getString(value_col));
				
			}
		}
		
		cursor.close();

		return values;
		
	}
	
	public void updateWPrefs (Map<String, String> update, int appWidgetId) {
		
		//database.update(DATABASE_TABLE4, update, KEY_W_ID+" = ?", new String[] { Integer.toString(appWidgetId) });
		
		
		Log.i(LOGTAG,"keyset size: "+update.size());
		
		//Set<String> keySet = update.keySet();
		Set<Entry<String, String>> entrySet = update.entrySet();
				
		for (Entry<String, String> entry : entrySet) {
			
			Log.i(LOGTAG,"updating preference: "+entry.getKey()+" with value: "+ entry.getValue()+" for widget: " + appWidgetId);
			
			String sql = "DELETE FROM "+ DATABASE_TABLE4 + " WHERE "+KEY_W_ID+" = "+appWidgetId+" AND "+KEY_KEY+" = '"+entry.getKey()+"' ";
			
			Log.i(LOGTAG,"sql: "+sql);
			
			database.execSQL(sql);
			
			sql = "REPLACE INTO "+ DATABASE_TABLE4 + " ("+KEY_W_ID+","+KEY_KEY+","+KEY_VALUE+") VALUES ('"+appWidgetId+"','"+entry.getKey()+"','"+entry.getValue()+"') ";
			
			Log.i(LOGTAG,"sql: "+sql);
			
			database.execSQL(sql);

		}
		
		
		/*
		for (String key : update.keySet()) {
			
			String value = (String) update.get(key);
			
			database.execSQL("REPLACE INTO "+ DATABASE_TABLE4 + " ("+KEY_W_ID+","+KEY_KEY+","+KEY_VALUE+") VALUES ('"+appWidgetId+"','"+key+"','"+value+"');");
			
		}
		*/
		
	}
	
	/**
	 * Fetches history, last 20 entries
	 * 	  
	 
	public Cursor fetchHistory() {
		Cursor cursor = database.rawQuery("SELECT "+KEY_ROWID+", "+KEY_PATH+", "+KEY_URI_ID+" FROM "+DATABASE_TABLE2+" LIMIT 20",null);
		
		Log.i(LOGTAG,"fetchhistory cursor count: "+cursor.getCount());
		
		return cursor;
		
		//return database.query(DATABASE_TABLE, new String[] { KEY_ROWID, KEY_URI, KEY_FLAGS}, null, null, null, null, null);
	}
	
	*/
	
	/**
	 * Return a Cursor positioned at the defined wallpaper
	 
	public Cursor fetch(String path) throws SQLException {
		Cursor mCursor = database.query(true, DATABASE_TABLE1, new String[] {
				KEY_ROWID, KEY_PATH, KEY_URI_ID },
				KEY_PATH + "=" + path, null, null, null, null, null);
		if (mCursor != null) {
			mCursor.moveToFirst();
		}
		return mCursor;
	}
	*/
	
	/**
	 * Return a Cursor positioned at the defined wallpaper
	 */
	public String fetchRandom(int appWidgetId) throws SQLException {
		
		Cursor cursor = database.rawQuery("SELECT "+KEY_ROWID+", "+KEY_PATH+","+KEY_URI_ID+" FROM "+DATABASE_TABLE1+"",null);
		int c = cursor.getCount();
		Random generator = new Random();
		int r = 0;
		
		if(c>0){
			r = generator.nextInt(c);
		}else{
			return null;
		}
		
		cursor.moveToPosition(r);
		
		int col = cursor.getColumnIndex(KEY_PATH);
		current_wp = cursor.getString(col);
		
		current_wp = URLDecoder.decode(current_wp);
		
		col = cursor.getColumnIndex(KEY_URI_ID);
		String current_uri = cursor.getString(col);
		
		//col = cursor.getColumnIndex(KEY_URI_ID);
		//current_id = cursor.getString(col);
		
		addHistory(appWidgetId, current_wp, current_uri);
		
		return current_wp;
		
	}
	
	
	public String fetchNext(int appWidgetId) throws SQLException {
		
		//String current_wp = getCurrentWP();
		
		current_wp = null;
		int current_row = 0;
		//String current_uri = null;
		
		Cursor cursor_last = database.rawQuery("SELECT "+KEY_ROWID+", "+KEY_PATH+", "+KEY_URI_ID+" FROM "+DATABASE_TABLE2+" WHERE "+KEY_W_ID+" = '"+appWidgetId+"' ORDER BY "+KEY_ROWID+" DESC LIMIT 1",null);
		
		if(cursor_last.getCount()>0){
			cursor_last.moveToPosition(0);
			
			int col = cursor_last.getColumnIndex(KEY_PATH);
			cursor_last.getString(col);
			
			current_wp = cursor_last.getString(col);
			
			//current_wp = URLDecoder.decode(current_wp);
			
			/*
			col = cursor_last.getColumnIndex(KEY_URI_ID);
			cursor_last.getString(col);
			
			//current_uri = cursor_last.getString(col);
			*/
		}
		
		cursor_last.close();
		
		Log.i(LOGTAG,"current_wp: "+current_wp+" current_row:"+current_row);
		
		if (current_wp == null){
			Cursor cursor_next = database.rawQuery("SELECT "+KEY_ROWID+", "+KEY_PATH+", "+KEY_URI_ID+" FROM "+DATABASE_TABLE1+" ORDER BY "+KEY_ROWID+" ASC LIMIT 1",null);
			
			if(cursor_next.getCount()<=0){
				return null;
			}
			
			cursor_next.moveToPosition(0);
			int col = cursor_next.getColumnIndex(KEY_PATH);
			String next_wp = cursor_next.getString(col);
			
			next_wp = URLDecoder.decode(next_wp);
			
			col = cursor_next.getColumnIndex(KEY_URI_ID);
			String next_uri = cursor_next.getString(col);
			
			cursor_next.close();
			
			addHistory(appWidgetId, next_wp, next_uri);
			return next_wp;
		}else{
			
			Cursor row_cursor = database.rawQuery("SELECT "+KEY_ROWID+", "+KEY_PATH+", "+KEY_URI_ID+" FROM "+DATABASE_TABLE1+" WHERE "+KEY_PATH+" = '"+current_wp+"' ",null);
			
			if(row_cursor.getCount()<=0){
				
				row_cursor.close();
				
				Cursor cursor = database.rawQuery("SELECT "+KEY_ROWID+", "+KEY_PATH+", "+KEY_URI_ID+" FROM "+DATABASE_TABLE1+" ORDER BY "+KEY_ROWID+" ASC LIMIT 1",null);
    			
    			if(cursor.getCount()<=0){
    				return null;
    			}
    			
    			cursor.moveToPosition(0);
    			
    			int col = cursor.getColumnIndex(KEY_PATH);
    			String next_wp = cursor.getString(col);
    			
    			next_wp = URLDecoder.decode(next_wp);
    			
    			col = cursor.getColumnIndex(KEY_URI_ID);
    			String next_uri = cursor.getString(col);
    			
    			cursor.close();
    			
    			addHistory(appWidgetId, next_wp, next_uri);
    			
    			return next_wp;
				
			}else{
				
				row_cursor.moveToPosition(0);
				int col = row_cursor.getColumnIndex(KEY_ROWID);
				row_cursor.getString(col);
				current_row = row_cursor.getInt(col);
				row_cursor.close();
				
	    		Cursor cursor = database.rawQuery("SELECT "+KEY_ROWID+", "+KEY_PATH+", "+KEY_URI_ID+" FROM "+DATABASE_TABLE1+" WHERE "+KEY_ROWID+" > "+current_row+" ORDER BY "+KEY_ROWID+" ASC LIMIT 1",null);
	    		
	    		if(cursor.getCount()<=0){
	    			cursor = database.rawQuery("SELECT "+KEY_ROWID+", "+KEY_PATH+", "+KEY_URI_ID+" FROM "+DATABASE_TABLE1+" ORDER BY "+KEY_ROWID+" ASC LIMIT 1",null);
	    			
	    			if(cursor.getCount()<=0){
	    				return null;
	    			}
	    			
	    			cursor.moveToPosition(0);
	    			
	    			col = cursor.getColumnIndex(KEY_PATH);
	    			String next_wp = cursor.getString(col);
	    			
	    			next_wp = URLDecoder.decode(next_wp);
	    			
	    			col = cursor.getColumnIndex(KEY_URI_ID);
	    			String next_uri = cursor.getString(col);
	    			
	    			cursor.close();
	    			
	    			addHistory(appWidgetId, next_wp, next_uri);
	    			
	    			return next_wp;
	    		}else{
	    			cursor.moveToPosition(0);
	        		col = cursor.getColumnIndex(KEY_PATH);
	        		String next_wp = cursor.getString(col);
	    			
	        		next_wp = URLDecoder.decode(next_wp);
	        		
	    			col = cursor.getColumnIndex(KEY_URI_ID);
	    			String next_uri = cursor.getString(col);
	    			
	    			cursor.close();
	    			
	        		addHistory(appWidgetId, next_wp, next_uri);
	    			return next_wp;
	    		}
    		
			}
		}
	}
	
	public String fetchLast(int appWidgetId) {
		Cursor cursor = database.rawQuery("SELECT "+KEY_ROWID+", "+KEY_PATH+" FROM "+DATABASE_TABLE2+" WHERE "+KEY_W_ID+" = '"+appWidgetId+"' ORDER BY "+KEY_ROWID+" DESC LIMIT 1",null);
		
		if (cursor.getCount()<=0){	
			
			return null;
			
		}
		
		cursor.moveToPosition(0);
		
		int id_col = cursor.getColumnIndex(KEY_ROWID);
		
		Log.i(LOGTAG,"_id column id: "+id_col);
		
		int col = cursor.getColumnIndex(KEY_PATH);
		current_wp = cursor.getString(col);
		
		current_wp = URLDecoder.decode(current_wp);
		
		Log.i(LOGTAG,"current_wp: "+current_wp);
		
		//addHistory(current_wp, null);
		
		return current_wp;
		
	}
	
	public void add(String path, String uri_id) {
		
		//return database.insert(DATABASE_TABLE, null, createContentValues(path, 1)); 
		if(path != null && uri_id != null){
			
			final String encodedPath = URLEncoder.encode(path);
		
			database.execSQL("REPLACE INTO "+ DATABASE_TABLE1 + " ("+KEY_PATH+","+KEY_URI_ID+") VALUES ('"+encodedPath+"','"+uri_id+"');");
		
		}
		
		return;
	}
	
	public void addAll(Cursor c){
		
		int count = c.getCount(); 
		int image_column_index = c.getColumnIndex(KEY_URI_ID);
        int dataColumnIndex = c.getColumnIndex(KEY_PATH);
        int bucket_column_index = c.getColumnIndex(KEY_FOLDER);
        //int uri_column_index = image_column_index;
        
        for(int i = 0; i < count; i++){
        	c.moveToPosition(i);
        	String path = c.getString(dataColumnIndex);
        	String uri_id = c.getString(image_column_index);
        	String folder = c.getString(bucket_column_index);
        	
        	if(path != null && uri_id != null){
        		
        		final String encodedPath = URLEncoder.encode(path);
        		final String encodedFolder = URLEncoder.encode(folder);
        		
    			database.execSQL("REPLACE INTO "+ DATABASE_TABLE1 + " ("+KEY_PATH+","+KEY_URI_ID+","+KEY_FOLDER+") VALUES ('"+encodedPath+"','"+uri_id+"','"+encodedFolder+"');");
    			
    			Log.i("androframe","adding entry with path:"+encodedPath+" and uri:"+uri_id+" and folder:"+folder);
    			
    		}
        }
    }
	
	
	public void add(String path, String uri_id, String folder) {
		
		//return database.insert(DATABASE_TABLE, null, createContentValues(path, 1)); 
		
		Log.i(LOGTAG,"dbservice.add() path:"+path+" uri_id:"+uri_id+" folder:"+folder);
		
		if(path != null && uri_id != null){
			
			final String encodedPath = URLEncoder.encode(path);
			final String encodedFolder = URLEncoder.encode(folder);
			
			database.execSQL("REPLACE INTO "+ DATABASE_TABLE1 + " ("+KEY_PATH+","+KEY_URI_ID+","+KEY_FOLDER+") VALUES ('"+encodedPath+"','"+uri_id+"','"+encodedFolder+"');");
		
		}
		
		return;
	}
	
	public void addHistory(int appWidgetId, String path, String uri_id) {
		
		//return database.insert(DATABASE_TABLE, null, createContentValues(path, 1)); 
		
		if(path != null && uri_id != null){
			final String encodedPath = URLEncoder.encode(path);
			
			database.execSQL("INSERT INTO "+ DATABASE_TABLE2 + " ("+KEY_PATH+","+KEY_URI_ID+","+KEY_W_ID+") VALUES ('"+encodedPath+"','"+uri_id+"','"+appWidgetId+"');");
		
		}
		
		return;
	}
	
	public void dropTables() {
		
		//return database.insert(DATABASE_TABLE, null, createContentValues(path, 1)); 
		
		dbHelper.dropTables();
		
		return;
	}
	
	
	private class DatabaseHelper extends SQLiteOpenHelper {
		
		private static final String DATABASE_NAME = "androframe";

		private static final int DATABASE_VERSION = 6;
	    
		// Database creation sql statement
		private static final String DATABASE_CREATE1 = "create table "+DATABASE_TABLE1+" (_id integer primary key autoincrement, path text not null unique, uri_id text not null, folder text)";
		private static final String DATABASE_CREATE2 = "create table "+DATABASE_TABLE2+" (_id integer primary key autoincrement, w_id integer not null, path text not null, uri_id text)";
		private static final String DATABASE_CREATE3 = "create table "+DATABASE_TABLE3+" (setting_key text not null, setting_value text)";
		private static final String DATABASE_CREATE4 = "create table "+DATABASE_TABLE4+" (w_id integer not null, setting_key text not null, setting_value text)";
		private static final String DATABASE_CREATE5 = "create table "+DATABASE_TABLE5+" (path text not null unique, setting_key text not null, setting_value text)";
		private static final String DATABASE_CREATE6 = "create table "+DATABASE_TABLE6+" (folder text not null)";
		
		
		public DatabaseHelper(Context context) {
			super(context, DATABASE_NAME, null, DATABASE_VERSION);
		}

		// Method is called during creation of the database
		@Override
		public void onCreate(SQLiteDatabase database) {
			database.execSQL(DATABASE_CREATE1);
			database.execSQL(DATABASE_CREATE2);
			database.execSQL(DATABASE_CREATE3);
			database.execSQL(DATABASE_CREATE4);
			database.execSQL(DATABASE_CREATE5);
			database.execSQL(DATABASE_CREATE6);
		}

		// Method is called during an upgrade of the database, e.g. if you increase
		// the database version
		@Override
		public void onUpgrade(SQLiteDatabase database, int oldVersion,
				int newVersion) {
			Log.w(DatabaseHelper.class.getName(),
					"Upgrading database from version " + oldVersion + " to "
							+ newVersion + ", which will destroy all old data");
			database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE1+"");
			database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE2+"");
			database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE3+"");
			database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE4+"");
			database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE5+"");
			database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE6+"");
			onCreate(database);
		}
		
		public void dropTables() {
    		
    		//return database.insert(DATABASE_TABLE, null, createContentValues(path, 1)); 
    		
    		database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE1+"");
    		database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE2+"");
    		database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE3+"");
    		database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE4+"");
    		database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE5+"");
    		database.execSQL("DROP TABLE IF EXISTS "+DATABASE_TABLE6+"");
    		
    		database.execSQL(DATABASE_CREATE1);
			database.execSQL(DATABASE_CREATE2);
			database.execSQL(DATABASE_CREATE3);
			database.execSQL(DATABASE_CREATE4);
			database.execSQL(DATABASE_CREATE5);
			database.execSQL(DATABASE_CREATE6);
    		
    		return;
    	}
	}
	
	
}