package com.channon.android.androframe_ads.objects;

import java.net.URLDecoder;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.provider.BaseColumns;
import android.provider.MediaStore;
import android.provider.MediaStore.Images.ImageColumns;
import android.provider.MediaStore.MediaColumns;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.GridView;
import android.widget.ImageView;

import com.channon.android.androframe_ads.AndroFrame;
import com.channon.android.androframe_ads.R;

public class MediaChooserActivity extends Activity{
	private final String LOGTAG = "com.channon.android.mediachooser.MediaChooserActivity";
	
	DatabaseService dbService;
	public ImageCursor imagecursor;
	Bundle c = new Bundle();
	GridView gView;
	
	SharedPreferences sp;
	
	String action;
	
	int ITEMS_PER_PAGE = 30;
  
	@Override
	public void onCreate(Bundle savedInstanceState) {
	        super.onCreate(savedInstanceState);

	        //loads custom gallery layout, may need to take place after image collection
	        setContentView(R.layout.imagechooser);
	        
	        sp = PreferenceManager.getDefaultSharedPreferences(AndroFrame.getInstance());
	        
	        try{
	        	String ipp = sp.getString("thumbs_per_page", "30");
	        	
	        	int tmp;
	        	
	        	try{
	        		tmp = Integer.parseInt(ipp);
	        	}catch(NumberFormatException e){
	        		tmp = 30;
	        	}
	        	
	        	ITEMS_PER_PAGE = tmp;
	        	
	        	Log.i(LOGTAG, "ITEMS_PER_PAGE loaded from prefs: "+ITEMS_PER_PAGE);
	        }catch (ClassCastException  e){
	        	ITEMS_PER_PAGE = 30;
	        	Log.i(LOGTAG, "ITEMS_PER_PAGE exception: "+ITEMS_PER_PAGE);
	        }
	        
	        dbService = ((AndroFrame) getApplication()).getDBService();

	        Intent intent = getIntent();

	        c=intent.getExtras();
	        
	        action = intent.getAction();

	        imagecursor = new ImageCursor(this, c.getString("bucket"), action);



	        Button btnUp = (Button)findViewById(R.id.btnUp);
	        btnUp.setOnClickListener (new OnClickListener() {
	        	public void onClick(View arg0) {
	        		Log.i(LOGTAG, "btnUp onclick");
	        		
	        		if(imagecursor.prevPage()){
	        			Log.i(LOGTAG, "prev page loaded");
	        			//gView.setAdapter(new ImageAdapter(getParent()));

	        		}

	        	}
	        });
	        
	        Button selectAllBtn = (Button)findViewById(R.id.selectAllBtn);
	        selectAllBtn.setOnClickListener (new OnClickListener() {
	        	public void onClick(View arg0) {
	        		Log.i(LOGTAG, "selectAllBtn onclick");
	        		
	        		imagecursor.selectAll();

	        	}
	        });

	        Button btnDown = (Button)findViewById(R.id.btnDown);
	        btnDown.setOnClickListener (new OnClickListener() {
	        	public void onClick(View arg0) {
	        		Log.i(LOGTAG, "btnDown onclick");
	        		
	        		if(imagecursor.nextPage()){
	        			
	        			Log.i(LOGTAG, "next page loaded");
	        			//gView.setAdapter(new ImageAdapter(getParent()));

	        		}

	        	}
	        });

	        Button btnSelect = (Button)findViewById(R.id.selectBtn);
	        if(action.equals("REMOVE")){
	        	btnSelect.setText(R.string.remove);
	        }else{
	        	btnSelect.setText(R.string.submit);
	        }
	        
	        btnSelect.setOnClickListener(new OnClickListener() {
				public void onClick(View arg0) {
					Log.i(LOGTAG, "select onclick start");
					
					finishUp();
					
				}	        	
	        });

	}
	
	protected void finishUp() {
		
		Log.i(LOGTAG, "finishUp");
		Set<Integer> positions = imagecursor.items.keySet();
		
		for (Integer i : positions) {
			ChooserItem item = imagecursor.items.get(i);
			
			Log.i(LOGTAG, "thmb selection: "+item.selected+" action:"+action);
        	
        	if(item.selected){
        		Log.i(LOGTAG, "in the loop");
        		//imagecursor.imagecursor.moveToPosition(i);

        		if(action.equals("REMOVE")){
        			dbService.delete(item.path);
        		}else{
        			dbService.add(item.path,item.uri,item.bucket);
        		}

        	}
        }
		
		finish();
		
	}

	@Override
	public void onDestroy(){
		
		//imagecursor.imagecursor.close();
		super.onDestroy();
	}
	
	@Override 
	public void onResume(){
		
		sp = PreferenceManager.getDefaultSharedPreferences(AndroFrame.getInstance());
        
		try{
        	String ipp = sp.getString("thumbs_per_page", "30");
        	
        	int tmp;
        	
        	try{
        		tmp = Integer.parseInt(ipp);
        	}catch(NumberFormatException e){
        		tmp = 30;
        	}
        	
        	ITEMS_PER_PAGE = tmp;
        	
        	Log.i(LOGTAG, "ITEMS_PER_PAGE loaded from prefs: "+ITEMS_PER_PAGE);
        }catch (ClassCastException  e){
        	ITEMS_PER_PAGE = 30;
        	Log.i(LOGTAG, "ITEMS_PER_PAGE exception: "+ITEMS_PER_PAGE);
        }
		
		super.onResume();
	}
	

	class ImageCursor {
		private Cursor imagecursor;
		public int count;
		//public Bitmap[] thumbnails;
	    //public boolean[] thumbnailsselection;
	    //public String[] arrPath;
	    
	    public int image_column_index;
	    public int dataColumnIndex;
	    public int uri_column_index;
	    public int bucket_column_index;
	    
	    public Map<Integer, ChooserItem> items = new HashMap<Integer, ChooserItem>();
	    
	    ProgressDialog progress;
	    ThumbLoader tl;
	    
	    private int NUM_PAGES;
	    private int currentPage = 1;
	    private int currentPageItems=0;

	    // Database fields
    	public static final String KEY_ROWID = "_id";
    	public static final String KEY_PATH = "path";
    	public static final String KEY_URI_ID = "uri_id";
    	public static final String KEY_FOLDER = "folder";

    	//List Choices
    	public static final int CHOICE_CURRENT = 1;
    	public static final int CHOICE_INTERNAL = 4;
    	public static final int CHOICE_EXTERNAL = 5;
    	
		public ImageCursor(Context context, String folder, String action){

			imagecursor = null;
			final String[] columns = { MediaColumns.DATA, BaseColumns._ID, ImageColumns.BUCKET_DISPLAY_NAME };
            final String orderBy = BaseColumns._ID;
            
            Log.i(LOGTAG, "folder: "+folder+" action:"+action);
            
           // String[] choices = new String[1];
           // choices[0] = string;
            
           if(action.equals("REMOVE")){
        	   DatabaseService dbService = ((AndroFrame) getApplicationContext()).getDBService();
        	   imagecursor = dbService.fetchFolder(c.getString("bucket"));
        	   startManagingCursor(imagecursor);
        	   
        	   
        	   if(imagecursor!=null){
               	   count = imagecursor.getCount(); 
               	   image_column_index = imagecursor.getColumnIndex(KEY_URI_ID);
                   dataColumnIndex = imagecursor.getColumnIndex(KEY_PATH);
                   bucket_column_index = imagecursor.getColumnIndex(KEY_FOLDER);
                   uri_column_index = image_column_index;
               }else {
               	count = 0;
               }
           }else{
        	   imagecursor = managedQuery(
                       MediaStore.Images.Media.EXTERNAL_CONTENT_URI, columns, ImageColumns.BUCKET_DISPLAY_NAME+"='"+folder+"'",
                       null, orderBy);
        	   
        	   if(imagecursor!=null){
               	   count = imagecursor.getCount(); 
               	   image_column_index = imagecursor.getColumnIndex(BaseColumns._ID);
                   dataColumnIndex = imagecursor.getColumnIndex(MediaColumns.DATA);
                   bucket_column_index = imagecursor.getColumnIndex(ImageColumns.BUCKET_DISPLAY_NAME);
                   uri_column_index = image_column_index;
               }else {
               	count = 0;
               }
           }

            
            Log.i(LOGTAG, "count:"+count);
            NUM_PAGES = Math.round(count/ITEMS_PER_PAGE) + 1;
            Log.i(LOGTAG, "NUM_PAGES:"+NUM_PAGES);
            if(count < ITEMS_PER_PAGE){
            	currentPageItems = count; 
            }else{
            	currentPageItems = ITEMS_PER_PAGE;
            }
            
            for (int i = 0; i<count; i++){
            	ChooserItem item = new ChooserItem();
            	imagecursor.moveToPosition(i);
            	item.ID = imagecursor.getInt(image_column_index);
            	item.bucket = imagecursor.getString(bucket_column_index);
            	item.bucket = URLDecoder.decode(item.bucket);
            	item.path = imagecursor.getString(dataColumnIndex);
            	item.path = URLDecoder.decode(item.path);
            	item.uri = imagecursor.getString(uri_column_index);
            	items.put(i, item);
            }
            
            progress = new ProgressDialog(context);
            int start = 1;
            if(count<start) count = start;
            
            progress.setMessage("Loading image ("+start+") through ("+currentPageItems+") of ("+count+") Images...");
            progress.setCancelable(false);
            
            tl = new ThumbLoader(progress, items);
            tl.execute();

            
		}

		public void handleReturn(Map<Integer, ChooserItem> items){
			
			this.items = items;
			
			gView = (GridView)findViewById(R.id.gview);
	        gView.setAdapter(new ImageAdapter(getBaseContext()));

	        gView.setOnItemClickListener(new OnItemClickListener() {
	            public void onItemClick(AdapterView<?> parent, View v, int position,
	                    long id) {


	            }

	        });
		}

		public class ThumbLoader extends AsyncTask<Void, Void, Void> {
			  private ProgressDialog progress;
			  public Map<Integer, ChooserItem> items = new HashMap<Integer, ChooserItem>();
			  
			public ThumbLoader(ProgressDialog progress, Map<Integer, ChooserItem> items) {
			    this.progress = progress;
			    this.items = items;
			  }

			  @Override
			public void onPreExecute() {
			    progress.show();
			    
			    //progress.show(context, title, message, indeterminate, cancelable, cancelListener);
			  }



			  @Override
			public void onPostExecute(Void unused) {
			    progress.dismiss();

			    handleReturn(this.items);
			  }

			@Override
			protected Void doInBackground(Void... params) {
				Log.v(LOGTAG,"doinginbackground");
				
				int start = ((currentPage-1)*ITEMS_PER_PAGE);
				int end = start + currentPageItems;
				
				Log.i(LOGTAG,"start:"+start+" end:"+end);

		            for (int i = start; i <= end; i++) {

		            	
		            	
		            	ChooserItem item = this.items.get(i);
		            	
		            	if(item == null) continue;
		            	
			        	int id = item.ID;
			        	Log.i(LOGTAG, "id: "+id);
			            
			            //arrPath[i]= imagecursor.getString(dataColumnIndex);
			            item.thumbnail = MediaStore.Images.Thumbnails.getThumbnail(
			                    getApplicationContext().getContentResolver(), id,
			                    MediaStore.Images.Thumbnails.MICRO_KIND, null);
			            
			            this.items.put(i, item);

			            Log.i(LOGTAG, "path: "+item.path+" thumb:"+item.thumbnail+" i:"+i+" bucket name:"+item.bucket);
			        	
			        }


				return null;
			}
		}
		
		public boolean selectAll(){
			
			int start = ((currentPage-1)*ITEMS_PER_PAGE);
			int end = start + currentPageItems - 1;
			
			Log.v(LOGTAG,"Select all items:"+start+" through:"+end);
			
			for(int i = start; i<=end;i++){
				
				ChooserItem item = this.items.get(i);
            	
            	if(item == null) continue;
            	
	        	int id = item.ID;
	        	Log.i(LOGTAG, "id: "+id);
	            
	            item.selected = true;
	            
	            this.items.put(i, item);

	            Log.i(LOGTAG, "path: "+item.path+" thumb:"+item.thumbnail+" i:"+i+" bucket name:"+item.bucket+" select: "+item.selected);
				
			}
			
			
			tl = new ThumbLoader(progress, items);
            tl.execute();
            
			return false;
			
		}

		public boolean nextPage(){

			Log.v(LOGTAG,"next page before:"+currentPage);

			if(NUM_PAGES>currentPage){
				currentPage++;

				if((currentPage*ITEMS_PER_PAGE) > count){
	            	currentPageItems = count-((currentPage-1)*ITEMS_PER_PAGE); 
	            }else{
	            	currentPageItems = ITEMS_PER_PAGE;
	            }

				Log.v(LOGTAG,"next page after:"+currentPage);
				
				int start = ((currentPage - 1) * ITEMS_PER_PAGE) + 1;
				int end = ((currentPage * ITEMS_PER_PAGE));
				
				progress.setMessage("Loading image ("+start+") through ("+end+") of ("+count+") Images...");progress.setCancelable(false);
	            
	            tl = new ThumbLoader(progress, items);
	            tl.execute();
				
				return true;

			}else{

				Log.v(LOGTAG,"next page no more pages");

				return false;
			}

		}

		public boolean prevPage(){

			Log.v(LOGTAG,"prev page before:"+currentPage);

			if(currentPage>1){
				currentPage--;
				
				if(count < ITEMS_PER_PAGE){
	            	currentPageItems = count; 
	            }else{
	            	currentPageItems = ITEMS_PER_PAGE;
	            }

				Log.v(LOGTAG,"prev page after:"+currentPage);
				
				int start = ((currentPage - 1) * ITEMS_PER_PAGE) + 1;
				int end = ((currentPage * ITEMS_PER_PAGE));
				
				progress.setMessage("Loading image ("+start+") through ("+end+") of ("+count+") Images...");
	            progress.setCancelable(false);
	            
	            tl = new ThumbLoader(progress, items);
	            tl.execute();
				
				return true;

			}else{

				Log.v(LOGTAG,"prev page no more pages");

				return false;
			}
		}

		public int getCurrentPage(){
			return currentPage;
		}

		public int getCurrentPageItems(){
			return currentPageItems;
		}

	}


	class ImageAdapter extends BaseAdapter{
	    //ImageView imageview;
	    //Context mContext;
	    //int columnIndex;
	    int currentPage;

	    private LayoutInflater mInflater;


	    public ImageAdapter(Context context){

	    	currentPage = imagecursor.getCurrentPage();

	        mInflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);

	    }

	    public ImageAdapter(OnClickListener onClickListener) {

	    }

	    public int getCount() {

	    	Log.v(LOGTAG,"getcurrpageitems:"+imagecursor.getCurrentPageItems());

	    	return imagecursor.getCurrentPageItems();
	    }

	    public Object getItem(int position) {
	        return (position+((currentPage-1)*ITEMS_PER_PAGE));
	    }

	    public long getItemId(int position) {
	        return (position+((currentPage-1)*ITEMS_PER_PAGE));
	    }

	    public View getView(int position, View convertView, ViewGroup parent) {

	    	int _offset = (currentPage-1)*ITEMS_PER_PAGE;
	    	int _position = _offset+position;
	    	//int _position = position;

	    	Log.i(LOGTAG, "creating viewholder and convertview w _position:"+_position);


	    	ViewHolder holder;
	    	//System.gc();

	    	if(convertView == null){
	         		holder = new ViewHolder();
					convertView = mInflater.inflate(
							R.layout.chooseritem, null);
					holder.imageview = (ImageView) convertView.findViewById(R.id.thumbImage);
					holder.checkbox = (CheckBox) convertView.findViewById(R.id.itemCheckBox);

					convertView.setTag(holder);
	            } else {

	            	holder = (ViewHolder) convertView.getTag();

	            }
	    	holder.checkbox.setId(_position);
			holder.imageview.setId(_position);

			Log.i(LOGTAG, "setting onclicks");

			holder.checkbox.setOnClickListener(new OnClickListener() {

				public void onClick(View v) {
					CheckBox cb = (CheckBox) v;
					int id = cb.getId();
					ChooserItem tmp = imagecursor.items.get(id);
					if (tmp.selected){
						cb.setChecked(false);
						tmp.selected = false;
						imagecursor.items.put(id, tmp);
					} else {
						cb.setChecked(true);
						tmp.selected = true;
						imagecursor.items.put(id, tmp);
					}
				}
			});
			holder.imageview.setOnClickListener(new OnClickListener() {

				public void onClick(View v) {
					int id = v.getId();
					Intent intent = new Intent();
					intent.setAction(Intent.ACTION_VIEW);
					intent.setDataAndType(Uri.parse("file://" + imagecursor.items.get(id).path), "image/*");
					startActivity(intent);
				}
			});

			holder.id = _position;
			ChooserItem item = imagecursor.items.get(holder.id);
			holder.imageview.setImageBitmap(item.thumbnail);
			holder.checkbox.setChecked(item.selected);

			Log.i(LOGTAG,"holder.id: "+holder.id+" position:"+position+" currentPage:"+currentPage+" thumb:"+item.thumbnail);

			return convertView;
	    }

    	@Override
	    public int getItemViewType (int position){

	    	if((position>(currentPage-1))&&(position<((currentPage*ITEMS_PER_PAGE)))){
	    		return getViewTypeCount();
	    	}else{
	    		return IGNORE_ITEM_VIEW_TYPE;
	    	}
	    }
    }
	
	@Override
	public void onBackPressed() {
	    // your code.
		
		showDialog(0);

		
	}
	
	@Override
	 protected Dialog onCreateDialog(int id) {
	       
	        	 return new AlertDialog.Builder(MediaChooserActivity.this)
	                .setIcon(R.drawable.alert_dialog_icon)
	                .setTitle(R.string.discard_changes)
	                .setMessage(R.string.info_discard_changes)
	                .setPositiveButton(R.string.alert_dialog_save, new DialogInterface.OnClickListener() {
	                    public void onClick(DialogInterface dialog, int whichButton) {
	                    	
	                		Intent resultValue = new Intent();
	            			//resultValue.putExtra(AppWidgetManager.EXTRA_APPWIDGET_ID, mAppWidgetId);
	            			setResult(RESULT_OK, resultValue);
	            			
	                        finish();
	                        
	                    }
	                })
	                .setNeutralButton(R.string.alert_dialog_cancel, new DialogInterface.OnClickListener() {
	                 public void onClick(DialogInterface dialog, int whichButton) {

	                 	return;
	             		
	                 }
	                })
	                .setNegativeButton(R.string.alert_dialog_exit, new DialogInterface.OnClickListener() {
	                 public void onClick(DialogInterface dialog, int whichButton) {

	                	Intent resultValue = new Intent();
	            		//resultValue.putExtra(AppWidgetManager.EXTRA_APPWIDGET_ID, mAppWidgetId);
	            		setResult(RESULT_OK, resultValue);
	            			
	                	finish();
	             		
	                 }
	                })
	                
	             .create();
		}
	
		

}