package com.channon.android.androframe_ads.objects;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.Log;

public class WidgetSharedPreferences implements SharedPreferences {

	protected final DatabaseService dbservice;
	//protected final String table;
	protected final int appWidgetId;

	public Map<String, String> values = new HashMap<String, String>();
	
	public WidgetSharedPreferences(DatabaseService db, int appWidgetId) {
		this.dbservice = db;
		//this.table = table;
		
		this.appWidgetId = appWidgetId;
		
		Log.i("androframe","widget shared preferences created for widget id: "+appWidgetId);
		
		this.initValues();
		
		this.cacheValues();
		
	}
	
	protected void cacheValues() {
		// fill a cursor and cache the values locally
		// this makes sure we dont have any floating cursor to dispose later

		
		Bundle tmp_values = dbservice.getWPrefs(appWidgetId);
		
		if(tmp_values.isEmpty()){
			return;
		}
			
		Set<String> keys = tmp_values.keySet();
		
		for(String key : keys){
			
			Log.i("androframe","key: "+key+" value: "+tmp_values.get(key));
			
			this.values.put(key, (String) tmp_values.get(key));
			
		}
		
		//this.values = dbservice.getWPrefs(appWidgetId);
		
		
		
		Log.i("androframe",values.size()+" default values retrieved for widget: "+appWidgetId);
		
		
		
	}
	
	protected void initValues() {
		
		String value = "category";
		String dummy1 = "control_category";
		
		this.values.put(dummy1, value);
		this.values.put("savePrefs", "category");
		this.values.put("first_category", "category");
		
		if(!values.containsKey("interval")) values.put("interval", "300");
		if(!values.containsKey("randomize")) values.put("randomize", "FALSE");
		
		this.values.put("second_category", "category");
		
		if(!values.containsKey("border")) values.put("border", "widget_frame_wood");
			
		
	}
	
	protected void stripValues() {
		
		values.remove("control_category");
		values.remove("savePrefs");
		values.remove("first_category");
		
		if(!values.containsKey("interval")) values.put("interval", "300");
		if(!values.containsKey("randomize")) values.put("randomize", "false");

		values.remove("second_category");
		
		if(!values.containsKey("border")) values.put("border", "widget_frame_wood");
		
	}
	
	public boolean contains(String key) {
		Log.v("androframe", "looking for key:" + key);
		
		return values.containsKey(key);
	}
	
	public class Editor implements SharedPreferences.Editor {
		
		//public ContentValues update = new ContentValues();
		
		public SharedPreferences.Editor clear() {
			Log.d(this.getClass().toString(), "clear()");
			//update = new ContentValues();
			return this;
		}

		public boolean commit() {
			Log.d(this.getClass().toString(), "commit() changes back to database");
			
			stripValues();
			
			dbservice.updateWPrefs(values, appWidgetId);
			
			// make sure we refresh the parent cached values
			cacheValues();
			
			// and update any listeners
			for(OnSharedPreferenceChangeListener listener : listeners) {
				listener.onSharedPreferenceChanged(WidgetSharedPreferences.this, null);
			}
			
			return true;
		}

		public android.content.SharedPreferences.Editor putBoolean(String key, boolean value) {
			return this.putString(key, Boolean.toString(value));
		}

		public android.content.SharedPreferences.Editor putFloat(String key, float value) {
			return this.putString(key, Float.toString(value));
		}

		public android.content.SharedPreferences.Editor putInt(String key, int value) {
			return this.putString(key, Integer.toString(value));
		}

		public android.content.SharedPreferences.Editor putLong(String key, long value) {
			return this.putString(key, Long.toString(value));
		}

		public android.content.SharedPreferences.Editor putString(String key, String value) {
			Log.d(this.getClass().toString(), String.format("Editor.putString(key=%s, value=%s)", key, value));
			values.put(key, value);
			
			for(OnSharedPreferenceChangeListener listener : listeners){
				
				listener.onSharedPreferenceChanged(WidgetSharedPreferences.this, key);
				
			}
			
			return this;
		}

		public android.content.SharedPreferences.Editor remove(String key) {
			Log.d(this.getClass().toString(), String.format("Editor.remove(key=%s)", key));
			values.remove(key);
			return this;
		}

		public void apply() {
			// TODO Auto-generated method stub
			
		}

		public android.content.SharedPreferences.Editor putStringSet(
				String arg0, Set<String> arg1) {
			// TODO Auto-generated method stub
			return null;
		}
		
	}


	public Editor edit() {
		Log.d(this.getClass().toString(), "edit()");
		return new Editor();
	}

	public Map<String, ?> getAll() {
		return values;
	}

	public boolean getBoolean(String key, boolean defValue) {
		return Boolean.valueOf(this.getString(key, Boolean.toString(defValue)));
	}

	public float getFloat(String key, float defValue) {
		return Float.valueOf(this.getString(key, Float.toString(defValue)));
	}

	public int getInt(String key, int defValue) {
		return Integer.valueOf(this.getString(key, Integer.toString(defValue)));
	}

	public long getLong(String key, long defValue) {
		return Long.valueOf(this.getString(key, Long.toString(defValue)));
	}

	public String getString(String key, String defValue) {
		Log.d(this.getClass().toString(), String.format("getString(key=%s, defValue=%s)", key, defValue));
		
		if(!values.containsKey(key)) return defValue;
		return values.get(key);
	}
	
	public List<OnSharedPreferenceChangeListener> listeners = new LinkedList<OnSharedPreferenceChangeListener>();

	public void registerOnSharedPreferenceChangeListener(OnSharedPreferenceChangeListener listener) {
		listeners.add(listener);
	}

	public void unregisterOnSharedPreferenceChangeListener(OnSharedPreferenceChangeListener listener) {
		listeners.remove(listener);
	}

	public Set<String> getStringSet(String arg0, Set<String> arg1) {
		// TODO Auto-generated method stub
		return null;
	}
	
}
