package com.channon.android.androframe_ads.widget;

import java.io.File;
import java.util.Set;

import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.RemoteViews;

import com.channon.android.androframe_ads.AndroFrame;
import com.channon.android.androframe_ads.R;
import com.channon.android.androframe_ads.objects.BitmapFunctions;
import com.channon.android.androframe_ads.objects.DatabaseService;
import com.channon.android.androframe_ads.objects.SwitchScheduler;

public class WidgetProvidor3x3 extends AppWidgetProvider{

	public static final int WIDGET_START = 0;
	public static final int WIDGET_NEXT = 1;
	public static String WIDGET_TOGGLE_FRAME = "ToggleFrame";
	public static final String SHOW="SHOW";
	public static final String HIDE="HIDE";
	
	/*
	static Map<String, Integer> borders = new HashMap<String, Integer>();
	static {
		borders.put("widget_frame_wood", R.drawable.widget_frame_wood);
		borders.put("widget_frame_metal", R.drawable.widget_frame_metal);
		borders.put("widget_frame_metalwave", R.drawable.widget_frame_metalwave);
		borders.put("widget_frame_jade", R.drawable.widget_frame_jade);
		borders.put("widget_frame_tiedye", R.drawable.widget_frame_tiedye);
		borders.put("widget_frame_flames", R.drawable.widget_frame_flames);
		borders.put("widget_frame_leaves", R.drawable.widget_frame_leaves);
		borders.put("widget_frame_moss", R.drawable.widget_frame_moss);
		borders.put("widget_frame_static", R.drawable.widget_frame_static);
		borders.put("widget_frame_water", R.drawable.widget_frame_water);
		borders.put("widget_frame_watercolor", R.drawable.widget_frame_watercolor);
		//borders.put("widget_frame_transparent", R.drawable.widget_frame_transparent);
		//borders.put("widget_frame_wood_hd", R.drawable.widget_frame_wood_hd);
		//borders.put("widget_frame_threaded_hd", R.drawable.widget_frame_threaded_hd);
			
	}
	*/
	
	public static final String defaultBorder = "widget_frame_wood";
	
	@Override
	public void onUpdate(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds) {
		Log.i("androframe", "widget onUpdate start");
		
		DatabaseService dbService = ((AndroFrame) context.getApplicationContext()).getDBService();
		
		final int N = appWidgetIds.length;

        // Perform this loop procedure for each App Widget that belongs to this provider
        for (int i=0; i<N; i++) {
            int appWidgetId = appWidgetIds[i];
            
            RemoteViews views = new RemoteViews(context.getPackageName(), R.layout.widget_layout);
            
            String path = dbService.fetchLast(appWidgetId);
			
			Log.i("androframe", "path:"+path);
			
			if(path!=null){
				File file = new File(path);
				
		        if(file.exists()){
					Log.i("androframe", "file currently exists");
					views.setImageViewBitmap(R.id.imageSwitcher1, BitmapFunctions.shrinkBitmap(path,100,225,225));
					views.setViewVisibility(R.id.warning, View.GONE);
		        }else{
		        	Log.i("androframe", "file does not exist");
		        	views.setViewVisibility(R.id.warning, View.VISIBLE);
		        	views.setTextViewText(R.id.warning, context.getString(R.string.loading));
		        }
	        }
			
            views = BitmapFunctions.addViews(context, views,appWidgetId,SHOW,225,225); 
            
            // Tell the AppWidgetManager to perform an update on the current app widget
            appWidgetManager.updateAppWidget(appWidgetId, views);
        }
        
        super.onUpdate(context, appWidgetManager, appWidgetIds);
        
        Log.i("androframe", "widget onUpdate end");
    }
		
	@Override
	public void onReceive(Context context, Intent intent) {
		
	 
		Log.i("androframe", "onReceive called");
		
		Bundle extras = intent.getExtras();
        int mAppWidgetId = 0;
        
        if (extras != null) {
            mAppWidgetId = extras.getInt(
                    AppWidgetManager.EXTRA_APPWIDGET_ID, 
                    AppWidgetManager.INVALID_APPWIDGET_ID);
            
            Set<String> keys = extras.keySet();
            
            for(String key : keys){
            	
            	 Log.i("androframe","extra key:"+key+" value:"+extras.get(key));
            	
            }
            
        }
        
        Log.i("androframe","for widget id: "+mAppWidgetId);
		
		AppWidgetManager appWidgetManager;
		appWidgetManager = AppWidgetManager
				.getInstance(context.getApplicationContext());
		
		ComponentName cn = new ComponentName(context.getApplicationContext(), WidgetProvidor3x3.class);
		
		int[] appWidgetIds = appWidgetManager.getAppWidgetIds(cn);
		
		final int N = appWidgetIds.length;

		Log.i("androframe", "appWidgetIds.length:"+N);
		
		onUpdate(context.getApplicationContext(), appWidgetManager, appWidgetIds);
		
		super.onReceive(context, intent);
	}
	
	public static RemoteViews toggleTaskbar(RemoteViews remoteView,Context context, int appWidgetId,String buttonPressed){
		
		 Log.i("androframe", "setButtons buttonPressed:"+buttonPressed+" appWidgetId:"+appWidgetId);
		 
		 if(buttonPressed.equals(SHOW)){
			 Log.i("androframe", "TOGGLE!");
			 
			 remoteView.setViewVisibility(R.id.taskbarLayout, View.VISIBLE);
			 return BitmapFunctions.addViews(context, remoteView, appWidgetId, HIDE,225,225);
	 	}else{
		 		
		 	remoteView.setViewVisibility(R.id.taskbarLayout, View.GONE);
		 	return BitmapFunctions.addViews(context, remoteView, appWidgetId, SHOW,225,225);
	 	}
		
		 
	}
	

	@Override
	public void onEnabled(Context context) {
		//This is called when an instance the App Widget is created for the first time. 
		//For example, if the user adds two instances of your App Widget, this is only called the first time. 
		//If you need to open a new database or perform other setup that only needs to occur once for all App Widget instances, 
		//then this is a good place to do it.
		
		Log.i("androframe", "widget onEnabled called");
		
        super.onEnabled(context);
		
	}
	
	@Override
	public void onDeleted(Context context, int[] appWidgetIds){
		//This is called every time an App Widget is deleted from the App Widget host.
		//int[] appWidgetIds	The appWidgetIds that have been deleted from their host.

		SwitchScheduler ss;
		ss = AndroFrame.getInstance().getSchedService();
		
		for (int id : appWidgetIds){
			ss.stop(id);
		}
		
		super.onDeleted(context, appWidgetIds);
	}
	
	@Override
	public void onDisabled(Context context) {
		//This is called when the last instance of your App Widget is deleted from the App Widget host. 
		//This is where you should clean up any work done in onEnabled(Context), such as delete a temporary database.
		
		SwitchScheduler ss;
		ss = AndroFrame.getInstance().getSchedService();
		
		AppWidgetManager appWidgetManager;
		appWidgetManager = AppWidgetManager
				.getInstance(context.getApplicationContext());
		
		ComponentName cn = new ComponentName(context.getApplicationContext(), WidgetProvidor3x3.class);
		
		int[] appWidgetIds = appWidgetManager.getAppWidgetIds(cn);
		
		for (int id : appWidgetIds){
			ss.stop(id);
		}
		
		super.onDisabled(context);

	}
	
	

}
